from time import sleep
from os import get_terminal_size
from .utils import StdoutHandler

stdout_handler = StdoutHandler()
stdout_handler.start()


RLL = "\x1b[1A\x1b[2K"  # Remove Last Line


def __validate_arguments(delay: float, sep: str, end: str) -> None:
    delay_type = type(delay)
    if not (delay_type is float or delay_type is int):
        raise TypeError("Argument 'delay' must be a float")
    if type(sep) is not str:
        raise TypeError("Argument 'sep' must be a string")
    if type(end) is not str:
        raise TypeError("Argument 'end' must be a string")


def snail_print(*objects, delay: float = 0.1, sep: str = " ", end: str = "\n") -> None:
    __validate_arguments(delay, sep, end)
    try:
        height, _ = get_terminal_size()
    except OSError:
        height = 80

    prev_output = stdout_handler.get_last_output_line().strip(RLL)
    string = sep.join([str(i) for i in objects])
    string_len = len(string)

    for i in range(string_len):
        back_slash_n_count = string[0:i].count("\n")
        times_RLL = 1 + ((i - 1) // height) + back_slash_n_count
        string_frag = string[0 : i + 1]

        if i == 0 and prev_output:
            print(f"{RLL * times_RLL}{string_frag}")
        elif (i == string_len - 1) and end != "\n":
            print(f"{RLL * times_RLL}{prev_output + string_frag}", end=end)
        else:
            print(f"{RLL * times_RLL}{prev_output + string_frag}")
        sleep(delay)



import sys


# https://stackoverflow.com/questions/72575956/how-to-get-last-python-output
class StdoutHandler:
    def __init__(self):
        self.last_output = ""
        self.list_all_output = []

    def start(self):
        self._handled_stdout = sys.stdout
        sys.stdout = self

    def write(self, data: str):
        # write(data="") is called for the end kwarg in print(..., end="")
        if data:
            self.last_output = data
            self.list_all_output.append(data)
            self._handled_stdout.write(data)

    def end(self):
        sys.stdout = self._handled_stdout

    def flush(self):
        self._handled_stdout.flush()

    def get_last_output_line(self):
        s = "".join(self.list_all_output)
        last_line = s.rsplit("\n", 1)[-1]
        return last_line


if __name__ == "__main__":
    stdout_handler = StdoutHandler()
    stdout_handler.start()

    print("Hello World")
    last_output_line = stdout_handler.get_last_output_line()
    print(repr(last_output_line))

    print("Hello World", end="")
    last_output_line = stdout_handler.get_last_output_line()
    print(repr(last_output_line))

    print("Hello", "World", end="")
    last_output_line = stdout_handler.get_last_output_line()
    print(repr(last_output_line))



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['snail_print']

package_data = \
{'': ['*']}

setup_kwargs = {
    'name': 'snail-print',
    'version': '0.1.8',
    'description': 'A print funtion that slowly shows the output in console in real time',
    'long_description': '# Snail Print\n\n[![GitHub release][release-image]][release-url]\n[![codeclimate][codeclimate-image]][codeclimate-url]\n![Tests & Linter][ci-url]\n<a href="https://github.com/psf/black"><img alt="Code style: black" src="https://img.shields.io/badge/code%20style-black-000000.svg"></a>\n\nThis library includes a print funtion that slows down the showing of output in console, character by character, as though typed by a typewriter.\n\n\n## Getting started\n\nInstall the library with:\n\n```sh\npip install -U snail_print\n```\n\n### Usage\n\n![Presentation Demo Video](https://raw.githubusercontent.com/Baelfire18/snail_print/master/assets/presentacion_color.gif)\n\n*Gif 1: Example of use of this real time print library*\n\nMoreover, every console log in this video was made with the library and with [this file](https://github.com/Baelfire18/snail_print/master/presentation.py)\n\n## Documentation\n\n### snail_print\n\n```python\nfunction snail_print(*objects, delay=0.1, sep=" ", end="\\n")\n```\n\n#### Parameters\n\n+ `objects`: Can be any or even multiple python objects.\n\nThe objects whose string representation will be "snail printed".\n\n+ `delay`: `float` or `int`, default `0.1`.\n\nThe delay, measured in seconds, between the printing of each character in `object`.\n\n+ `sep`: `str`, default `" "`.\n\nIn case of having multiple arguments, one sep will be placed between each and the next.\n\n+ `end`: `str`, default `"\\n"`.\n\nThe final character of the print. By default creates a new line.\n\n\n## Testing\n\nRun the test suite with:\n\n```sh\npython -m unittest tests\n```\n\n## Install local\n\nTo install it locally from the source code:\n\n```sh\npython setup.py develop\n```\n\n[release-image]: https://img.shields.io/github/v/release/Baelfire18/snail_print.svg\n[release-url]: https://github.com/Baelfire18/snail_print/releases/latest\n[codeclimate-image]: https://codeclimate.com/github/Baelfire18/snail_print/badges/gpa.svg\n[codeclimate-url]: https://codeclimate.com/github/Baelfire18/snail_print\n[ci-url]: https://github.com/Baelfire18/snail_print/actions/workflows/tests-and-linter.yml/badge.svg\n',
    'author': 'Jose Antonio Castro',
    'author_email': 'jacastro18@uc.cl',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/Baelfire18/snail_print',
    'packages': packages,
    'package_data': package_data,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
